@@ -41,7 +41,6 @@ namespace GiNaC {
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(add, expairseq,
   print_func<print_context>(&add::do_print).
   print_func<print_latex>(&add::do_print_latex).
-  print_func<print_csrc>(&add::do_print_csrc).
   print_func<print_tree>(&add::do_print_tree).
   print_func<print_python_repr>(&add::do_print_python_repr))
 
@@ -189,48 +188,6 @@ void add::do_print_latex(const print_latex & c, unsigned level) const
 	print_add(c, level, true);
 }
 
-void add::do_print_csrc(const print_csrc & c, unsigned level) const
-{
-	if (precedence() <= level)
-		c.s << "(";
-	
-	// Print arguments, separated by "+" or "-"
-	char separator = ' ';
-        for (const auto & elem : seq) {
-		
-		// If the coefficient is negative, separator is "-"
-		if (elem.coeff.is_equal(_ex_1) || 
-			ex_to<numeric>(elem.coeff).numer().is_equal(*_num_1_p))
-			separator = '-';
-		c.s << separator;
-		if (elem.coeff.is_equal(_ex1) || elem.coeff.is_equal(_ex_1)) {
-			elem.rest.print(c, precedence());
-		} else if (ex_to<numeric>(elem.coeff).numer().is_one()
-                        or ex_to<numeric>(elem.coeff).numer().is_minus_one())
-		{
-			elem.rest.print(c, precedence());
-			c.s << '/';
-			ex_to<numeric>(elem.coeff).denom().print(c, precedence());
-		} else {
-			elem.coeff.print(c, precedence());
-			c.s << '*';
-			elem.rest.print(c, precedence());
-		}
-		
-		separator = '+';
-	}
-	
-	if (!overall_coeff.is_zero()) {
-		if (overall_coeff.info(info_flags::positive)
-		 || is_a<print_csrc_cl_N>(c) || !overall_coeff.info(info_flags::real))  // sign inside ctor argument
-			c.s << '+';
-		overall_coeff.print(c, precedence());
-	}
-		
-	if (precedence() <= level)
-		c.s << ")";
-}
-
 void add::do_print_python_repr(const print_python_repr & c, unsigned /*level*/) const
 {
 	c.s << class_name() << '(';
@@ -245,6 +202,9 @@ void add::do_print_python_repr(const print_python_repr & c, unsigned /*level*/)
 bool add::info(unsigned inf) const
 {
 	switch (inf) {
+		case info_flags::nonzero:
+                        return info(info_flags::positive)
+                                or info(info_flags::negative);
 		case info_flags::polynomial:
 		case info_flags::integer_polynomial:
 		case info_flags::cinteger_polynomial:
@@ -322,33 +282,31 @@ bool add::is_polynomial(const ex & var) const
 	return true;
 }
 
-int add::degree(const ex & s) const
+numeric add::degree(const ex & s) const
 {
-	int deg = std::numeric_limits<int>::min();
-	if (!overall_coeff.is_zero())
-		deg = 0;
-	
+	numeric deg(seq[0].rest.degree(s));
 	// Find maximum of degrees of individual terms
-        for (const auto & elem : seq) {
-		int cur_deg = elem.rest.degree(s);
-		if (cur_deg > deg)
-			deg = cur_deg;
+        for (const auto & elem : range(seq.begin()+1, seq.end())) {
+		const numeric& t = elem.rest.degree(s);
+                if (t > deg)
+                        deg = t;
 	}
+        if (deg.is_negative() and not overall_coeff.is_zero())
+                return *_num0_p;
 	return deg;
 }
 
-int add::ldegree(const ex & s) const
+numeric add::ldegree(const ex & s) const
 {
-	int deg = std::numeric_limits<int>::max();
-	if (!overall_coeff.is_zero())
-		deg = 0;
-	
+	numeric deg(seq[0].rest.ldegree(s));
 	// Find minimum of degrees of individual terms
-        for (const auto & elem : seq) {
-		int cur_deg = elem.rest.ldegree(s);
-		if (cur_deg < deg)
-			deg = cur_deg;
+        for (const auto & elem : range(seq.begin()+1, seq.end())) {
+		const numeric& t = elem.rest.ldegree(s);
+                if (t < deg)
+                        deg = t;
 	}
+        if (deg.is_positive() and not overall_coeff.is_zero())
+                return *_num0_p;
 	return deg;
 }
 
@@ -572,8 +530,8 @@ expair add::split_ex_to_pair(const ex & e) const
 {
 	if (is_exactly_a<mul>(e)) {
 		const mul &mulref(ex_to<mul>(e));
-		const ex &numfactor = mulref.overall_coeff;
-		if (numfactor.is_integer_one())
+		const numeric &numfactor = mulref.overall_coeff;
+		if (numfactor.is_one())
                         return expair(e, _ex1);
                 auto mulcopyp = new mul(mulref);
 		mulcopyp->overall_coeff = *_num1_p;
@@ -669,7 +627,7 @@ ex add::lead_coeff() const {
 ex add::combine_fractions() const
 {
         epvector rseq;
-        exmap map;
+        exmap fmap;
         ex oc = overall_coeff;
         for (const auto& pair : seq) {
                 const ex &e = recombine_pair_to_ex(pair);
@@ -677,11 +635,11 @@ ex add::combine_fractions() const
                         const power& p = ex_to<power>(e);
                         const ex& eexp = p.op(1);
                         if (eexp.info(info_flags::negative)) {
-                                auto it = map.find(p);
-                                if (it != map.end())
+                                auto it = fmap.find(p);
+                                if (it != fmap.end())
                                         it->second += _ex1;
                                 else
-                                        map[p] = _ex1;
+                                        fmap[p] = _ex1;
                         }
                         else
                                 rseq.push_back(pair);
@@ -697,13 +655,13 @@ ex add::combine_fractions() const
                         }
                         if (not denseq.empty()) {
                                 mul den(denseq);
-                                auto it = map.find(den);
-                                mul coeff(coseq, ex_to<numeric>(pair.coeff));
-                                if (it != map.end()) {
-                                        it->second += coeff;
+                                auto it = fmap.find(den);
+                                mul tcoeff(coseq, ex_to<numeric>(pair.coeff));
+                                if (it != fmap.end()) {
+                                        it->second += tcoeff;
                                 }
                                 else
-                                        map[den] = coeff;
+                                        fmap[den] = tcoeff;
                         }
                         else
                                 rseq.push_back(pair);
@@ -711,7 +669,7 @@ ex add::combine_fractions() const
                 else
                         rseq.push_back(pair);
         }
-        for (const auto& t : map)
+        for (const auto& t : fmap)
                 rseq.push_back(split_ex_to_pair(mul(t.first, t.second)));
         add rex(rseq);
         return add(rex, oc);
