@@ -22,6 +22,7 @@
 
 #include "basic.h"
 #include "ex.h"
+#include "ex_utils.h"
 #include "numeric.h"
 #include "power.h"
 #include "add.h"
@@ -139,40 +140,40 @@ void basic::print_dispatch(const registered_class_info & ri, const print_context
 	const registered_class_info * r_info = &ri;
 	const print_context_class_info * pc_info = &c.get_class_info();
 
-next_class:
-	const std::vector<print_functor> & pdt = r_info->options.get_print_dispatch_table();
-
-next_context:
+	const std::vector<print_functor> * pdt = &r_info->options.get_print_dispatch_table();
 	unsigned id = pc_info->options.get_id();
-	if (id >= pdt.size() || !(pdt[id].is_valid())) {
 
-		// Method not found, try parent print_context class
-		const print_context_class_info * parent_pc_info = pc_info->get_parent();
-		if (parent_pc_info != nullptr) {
-			pc_info = parent_pc_info;
-			goto next_context;
-		}
+        while(id >= pdt->size() or not ((*pdt)[id].is_valid())) {
 
-		// Method still not found, try parent class
-		const registered_class_info * parent_reg_info = r_info->get_parent();
-		if (parent_reg_info != nullptr) {
-			r_info = parent_reg_info;
-			pc_info = &c.get_class_info();
-			goto next_class;
-		}
+                // Method not found, try parent print_context class
+                const print_context_class_info * parent_pc_info = pc_info->get_parent();
+                if (parent_pc_info != nullptr) {
+                        pc_info = parent_pc_info;
+                        id = pc_info->options.get_id();
+                        continue;
+                }
 
-		// Method still not found. This shouldn't happen because basic (the
-		// base class of the algebraic hierarchy) registers a method for
-		// print_context (the base class of the print context hierarchy),
-		// so if we end up here, there's something wrong with the class
-		// registry.
-		throw (std::runtime_error(std::string("basic::print(): method for ") + class_name() + "/" + c.class_name() + " not found"));
+                // Method still not found, try parent class
+                const registered_class_info * parent_reg_info = r_info->get_parent();
+                if (parent_reg_info != nullptr) {
+                        r_info = parent_reg_info;
+                        pc_info = &c.get_class_info();
+                        pdt = &r_info->options.get_print_dispatch_table();
+                        id = pc_info->options.get_id();
+                        continue;
+                }
 
-	} else {
+                // Method still not found. This shouldn't happen because basic (the
+                // base class of the algebraic hierarchy) registers a method for
+                // print_context (the base class of the print context hierarchy),
+                // so if we end up here, there's something wrong with the class
+                // registry.
+                throw (std::runtime_error(std::string("basic::print(): method for ") + class_name() + "/" + c.class_name() + " not found"));
 
-		// Call method
-		pdt[id](*this, c, level);
-	}
+        }
+
+        // Call method
+        (*pdt)[id](*this, c, level);
 }
 
 /** Default output to stream. */
@@ -333,13 +334,13 @@ bool basic::is_polynomial(const ex & var) const
 }
 
 /** Return degree of highest power in object s. */
-int basic::degree(const ex & s) const
+numeric basic::degree(const ex & s) const
 {
 	return is_equal(ex_to<basic>(s)) ? 1 : 0;
 }
 
 /** Return degree of lowest power in object s. */
-int basic::ldegree(const ex & s) const
+numeric basic::ldegree(const ex & s) const
 {
 	return is_equal(ex_to<basic>(s)) ? 1 : 0;
 }
@@ -348,7 +349,7 @@ int basic::ldegree(const ex & s) const
 ex basic::coeff(const ex & s, const ex & n) const
 {
 	if (is_equal(ex_to<basic>(s)))
-		return n.is_integer_one() ? _ex1 : _ex0;
+		return n.is_one() ? _ex1 : _ex0;
 	else
 		return n.is_zero() ? *this : _ex0;
 }
@@ -380,7 +381,7 @@ ex basic::collect(const ex & s, bool distributed) const
 				ex key = _ex1;
                                 ex pre_coeff = xelem;
 				for (const auto & lelem : l) {
-					int cexp = pre_coeff.degree(lelem);
+					ex cexp = pre_coeff.degree(lelem);
 					pre_coeff = pre_coeff.coeff(lelem, cexp);
 					key *= pow(lelem, cexp);
 				}
@@ -410,10 +411,12 @@ ex basic::collect(const ex & s, bool distributed) const
 		}
 
 	} else {
-
 		// Only one object specified
-		for (int n=this->ldegree(s); n<=this->degree(s); ++n)
-			x += this->coeff(s,n)*power(s,n);
+                expairvec vec;
+                ex(*this).coefficients(s, vec);
+		for (const auto& term : vec)
+			x += term.first * power(s, term.second);
+                return x;
 	}
 	
 	// correct for lost fractional arguments and return
@@ -582,11 +585,10 @@ ex basic::subs_one_level(const exmap & m, unsigned options) const
 	exmap::const_iterator it;
 
         if (options & subs_options::no_pattern) {
-		ex thisex = *this;
-		it = m.find(thisex);
-		if (it != m.end())
-			return it->second;
-		return thisex;
+                ex thisex = *this;
+                for (const auto & pair : m)
+                        if (thisex.is_equal(pair.first))
+                                return pair.second;
 	} else {
                 for (const auto & elem : m) {
 			lst repl_lst;
