@@ -22,6 +22,7 @@
 
 #include <Python.h>
 #include "ex.h"
+#include "ex_utils.h"
 #include "symbol.h"
 #include "add.h"
 #include "mul.h"
@@ -316,21 +317,20 @@ bool ex::is_zero() const {
         return ex_to<numeric>(*this).is_zero();
 }
 
-bool ex::is_integer_one() const
+bool ex::is_one() const
 {
     if (!is_exactly_a<numeric>(*this))
         return false;
     const numeric& num = ex_to<numeric>(*this);
-    return num.is_integer() and num.is_one();
+    return num.is_one();
 }
   
-bool ex::is_integer_pmone() const
+bool ex::is_minus_one() const
 {
     if (!is_exactly_a<numeric>(*this))
         return false;
     const numeric& num = ex_to<numeric>(*this);
-    return num.is_integer()
-           and (num.is_one() or num.is_minus_one());
+    return num.is_minus_one();
 }
 
 void ex::set_domain(unsigned d)
@@ -535,7 +535,10 @@ static bool match_monom(const ex& term, const symbol& symb,
                                 numeric ee = ex_to<numeric>(mterm.op(1));
                                 if (ee.is_integer() and ee.to_int() > 1) {
                                         expairvec tmpvec;
-                                        expand(term).coefficients(symb, tmpvec);
+                                        ex e = expand(term);
+                                        if (not is_exactly_a<add>(e))
+                                                return false;
+                                        e.coefficients(symb, tmpvec);
                                         for (const auto& pair : tmpvec)
                                                 vec.push_back(std::make_pair(pair.first.subs(map),
                                                         pair.second.subs(map)));
@@ -556,6 +559,16 @@ static bool match_monom(const ex& term, const symbol& symb,
         return false;
 }
 
+numeric ex::degree(const ex & s) const
+{
+        return bp->degree(s);
+}
+
+numeric ex::ldegree(const ex & s) const
+{
+        return bp->ldegree(s);
+}
+
 /**
  * Return in vec a list of pairs with (coefficient, exponent) of ex
  * when interpreted as polynomial in s (with s any expression).
@@ -576,11 +589,6 @@ void ex::coefficients(const ex & s, expairvec & vec) const
         exmap submap {{s, symb}}, revmap {{symb, s}};
         ex sub = subs(submap);
 
-        if (has_nonposint_power(sub, symb)) {
-                vec.push_back(std::make_pair(*this, _ex0));
-                return;
-        }
-
         if (is_exactly_a<add>(sub)) {
                 const add& addref = ex_to<add>(sub);
                 const numeric& oc = addref.get_overall_coeff();
@@ -588,12 +596,14 @@ void ex::coefficients(const ex & s, expairvec & vec) const
                         vec.push_back(std::make_pair(oc, _ex0));
                 for (const auto& term : addref.seq) {
                         ex tmp = addref.recombine_pair_to_ex(term);
-                        if (not match_monom(tmp, symb, vec, revmap))
+                        if (has_nonposint_power(tmp, symb)
+                            or not match_monom(tmp, symb, vec, revmap))
                                 vec.push_back(std::make_pair(tmp.subs(revmap), _ex0));
                 }
         }
         else {
-                if (not match_monom(sub, symb, vec, revmap)) {
+                if (has_nonposint_power(sub, symb)
+                    or not match_monom(sub, symb, vec, revmap)) {
                         vec.clear();
                         vec.push_back(std::make_pair(*this, _ex0));
                 }
@@ -628,6 +638,16 @@ void ex::coefficients(const ex & s, expairvec & vec) const
 
 }
 
+ex ex::lcoeff(const ex & s) const
+{
+        return coeff(s, degree(s));
+}
+
+ex ex::tcoeff(const ex & s) const
+{
+        return coeff(s, ldegree(s));
+}
+
 ex ex::deep_combine_fractions(ex e)
 {
         if (is_a<expairseq>(e)) {
