@@ -20,12 +20,13 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <Python.h>
+#include "py_funcs.h"
 #include "function.h"
 #include "operators.h"
 #include "fderivative.h"
 #include "ex.h"
 #include "lst.h"
-#include "symmetry.h"
 #include "print.h"
 #include "power.h"
 #include "relational.h"
@@ -35,7 +36,6 @@
 #include "utils.h"
 #include "remember.h"
 #include "symbol.h"
-#include "py_funcs.h"
 
 #include <iostream>
 #include <string>
@@ -82,7 +82,7 @@ void function_options::initialize()
 	set_name(s1, s2);
 	nparams = 0;
 	eval_f = real_part_f = imag_part_f = conjugate_f = derivative_f
-		= expl_derivative_f = power_f = series_f = nullptr;
+            = pynac_eval_f = expl_derivative_f = power_f = series_f = nullptr;
 	evalf_f = nullptr;
 	evalf_params_first = true;
 	apply_chain_rule = true;
@@ -128,24 +128,28 @@ function_options & function_options::eval_func(eval_funcp_1 e)
 {
 	test_and_set_nparams(1);
 	eval_f = eval_funcp(e);
+        pynac_eval_f = eval_f;
 	return *this;
 }
 function_options & function_options::eval_func(eval_funcp_2 e)
 {
 	test_and_set_nparams(2);
 	eval_f = eval_funcp(e);
+        pynac_eval_f = eval_f;
 	return *this;
 }
 function_options & function_options::eval_func(eval_funcp_3 e)
 {
 	test_and_set_nparams(3);
 	eval_f = eval_funcp(e);
+        pynac_eval_f = eval_f;
 	return *this;
 }
 function_options & function_options::eval_func(eval_funcp_6 e)
 {
 	test_and_set_nparams(6);
 	eval_f = eval_funcp(e);
+        pynac_eval_f = eval_f;
 	return *this;
 }
 
@@ -319,6 +323,7 @@ function_options& function_options::eval_func(eval_funcp_exvector e)
 {
 	eval_use_exvector_args = true;
 	eval_f = eval_funcp(e);
+        pynac_eval_f = eval_f;
 	return *this;
 }
 function_options& function_options::evalf_func(evalf_funcp_exvector ef)
@@ -466,12 +471,6 @@ function_options & function_options::overloaded(unsigned o)
 	return *this;
 }
 
-function_options & function_options::set_symmetry(const symmetry & s)
-{
-	symtree = s;
-	return *this;
-}
-	
 void function_options::test_and_set_nparams(unsigned n)
 {
 	if (nparams==0) {
@@ -732,8 +731,8 @@ void function::print(const print_context & c, unsigned level) const
 			    << ", nops=" << nops()
 			    << std::endl;
 			unsigned delta_indent = static_cast<const print_tree &>(c).delta_indent;
-			for (size_t i=0; i<seq.size(); ++i)
-				seq[i].print(c, level + delta_indent);
+			for (auto term : seq)
+				term.print(c, level + delta_indent);
 			c.s << std::string(level + delta_indent, ' ') << "=====" << std::endl;
 		}
                 else {
@@ -794,16 +793,6 @@ next_context:
 				elem.print(c, level + delta_indent);
 			c.s << std::string(level + delta_indent, ' ') << "=====" << std::endl;
 
-		} else if (is_a<print_csrc>(c)) {
-
-			// Print function name in lowercase
-			std::string lname = opt.name;
-			size_t num = lname.size();
-			for (size_t i=0; i<num; i++)
-				lname[i] = tolower(lname[i]);
-			c.s << lname;
-			printseq(c, "(", ',', ")", exprseq::precedence(), function::precedence());
-
 		} else if (is_a<print_latex>(c)) {
 			c.s << opt.TeX_name;
 			printseq(c, "\\left(", ',', "\\right)", exprseq::precedence(), function::precedence());
@@ -853,23 +842,6 @@ ex function::eval(int level) const
 	GINAC_ASSERT(serial<registered_functions().size());
 	const function_options &opt = registered_functions()[serial];
 
-	// Canonicalize argument order according to the symmetry properties
-	if (seq.size() > 1 && !(opt.symtree.is_zero())) {
-		exvector v = seq;
-		GINAC_ASSERT(is_a<symmetry>(opt.symtree));
-		int sig = canonicalize(v.begin(), ex_to<symmetry>(opt.symtree));
-		if (sig != std::numeric_limits<int>::max()) {
-			// Something has changed while sorting arguments, more evaluations later
-			if (sig == 0)
-				return _ex0;
-			return ex(sig) * thiscontainer(v);
-		}
-	}
-
-	if (opt.eval_f==nullptr) {
-		return this->hold();
-	}
-
 	bool use_remember = opt.use_remember;
 	ex eval_result;
 	if (use_remember && lookup_remember_table(eval_result)) {
@@ -877,11 +849,20 @@ ex function::eval(int level) const
 	}
 	current_serial = serial;
 
-	if ((opt.python_func & function_options::eval_python_f) != 0u) {
+	if (opt.eval_f==nullptr)
+		return this->hold();
+        eval_funcp eval_f;
+        if (opt.pynac_eval_f == nullptr)
+                eval_f = opt.eval_f;
+        else
+                eval_f = opt.pynac_eval_f;
+
+	if (opt.pynac_eval_f == nullptr
+            and (opt.python_func & function_options::eval_python_f) != 0u) {
 		// convert seq to a PyTuple of Expressions
 		PyObject* args = py_funcs.exvector_to_PyTuple(seq);
 		// call opt.eval_f with this list
-		PyObject* pyresult = PyObject_CallMethod((PyObject*)opt.eval_f,
+		PyObject* pyresult = PyObject_CallMethod((PyObject*)eval_f,
 				const_cast<char*>("_eval_"), const_cast<char*>("O"), args);
 		Py_DECREF(args);
 		if (pyresult == nullptr) { 
@@ -898,21 +879,21 @@ ex function::eval(int level) const
 		}
 	}
 	else if (opt.eval_use_exvector_args)
-		eval_result = ((eval_funcp_exvector)(opt.eval_f))(seq);
+		eval_result = ((eval_funcp_exvector)(eval_f))(seq);
 	else
 	switch (opt.nparams) {
 		// the following lines have been generated for max. 14 parameters
 	case 1:
-		eval_result = ((eval_funcp_1)(opt.eval_f))(seq[1-1]);
+		eval_result = ((eval_funcp_1)(eval_f))(seq[1-1]);
 		break;
 	case 2:
-		eval_result = ((eval_funcp_2)(opt.eval_f))(seq[1-1], seq[2-1]);
+		eval_result = ((eval_funcp_2)(eval_f))(seq[1-1], seq[2-1]);
 		break;
 	case 3:
-		eval_result = ((eval_funcp_3)(opt.eval_f))(seq[1-1], seq[2-1], seq[3-1]);
+		eval_result = ((eval_funcp_3)(eval_f))(seq[1-1], seq[2-1], seq[3-1]);
 		break;
 	case 6:
-		eval_result = ((eval_funcp_6)(opt.eval_f))(seq[1-1], seq[2-1], seq[3-1], seq[4-1], seq[5-1], seq[6-1]);
+		eval_result = ((eval_funcp_6)(eval_f))(seq[1-1], seq[2-1], seq[3-1], seq[4-1], seq[5-1], seq[6-1]);
 		break;
 
 		// end of generated lines
@@ -1008,7 +989,7 @@ long function::calchash() const
 		v ^= this->op(i).gethash();
 	}
 
-	if ((flags & status_flags::evaluated) != 0u) {
+	if (is_evaluated()) {
 		setflag(status_flags::hash_calculated);
 		hashvalue = v;
 	}
@@ -1682,6 +1663,71 @@ bool has_symbol_or_function(const ex & x)
 	return false;
 }
 
+static bool has_oneof_function_helper(const ex& x,
+                const std::map<unsigned,int>& m)
+{
+	if (is_exactly_a<function>(x)
+            and m.find(ex_to<function>(x).get_serial()) != m.end())
+		return true;
+	for (size_t i=0; i<x.nops(); ++i)
+		if (has_oneof_function_helper(x.op(i), m))
+			return true;
+
+	return false;
+}
+
+static void has_allof_function_helper(const ex& x,
+                std::map<unsigned,int>& m)
+{
+	if (is_exactly_a<function>(x)) {
+                unsigned ser = ex_to<function>(x).get_serial();
+                if (m.find(ser) != m.end())
+        		m[ser] = 1;
+        }
+	for (size_t i=0; i<x.nops(); ++i)
+		has_allof_function_helper(x.op(i), m);
+}
+
+bool has_function(const ex& x,
+                const std::string& s)
+{
+        std::map<unsigned,int> m;
+        unsigned ser = 0;
+        for (const auto & elem : function::registered_functions()) {
+                if (s == elem.name)
+                        m[ser] = 0;
+                ++ser;
+        }
+        if (m.empty())
+                return false;
+        return has_oneof_function_helper(x, m);
+}
+
+bool has_function(const ex& x,
+                const std::vector<std::string>& v,
+                bool all)
+{
+        std::map<unsigned,int> m;
+        for (const auto & s : v) {
+                unsigned ser = 0;
+                for (const auto & elem : function::registered_functions()) {
+                        if (s == elem.name)
+                                m[ser] = 0;
+                        ++ser;
+                }
+        }
+        if (m.empty())
+                return false;
+        if (all) {
+                has_allof_function_helper(x, m);
+                for (const auto & p : m)
+                        // TODO: false negative if >1 func with same name
+                        if (p.second == 0)
+                                return false;
+                return true;
+        }
+        return has_oneof_function_helper(x, m);
+}
 
 } // namespace GiNaC
 
