@@ -7,6 +7,7 @@
 
 #include "inifcns.h"
 #include "function.h"
+#include "operators.h"
 #include "utils.h"
 
 namespace GiNaC {
@@ -32,13 +33,11 @@ static bool log_info(const function& f, unsigned inf)
                 return arg.info(info_flags::positive);
         case info_flags::positive:
                 return arg.info(info_flags::real) and
-                        is_exactly_a<numeric>(arg) and
-                        ex_to<numeric>(arg) > *_num1_p;
+                        (arg-_ex1).info(info_flags::positive);
         case info_flags::negative:
                 return arg.info(info_flags::real) and
-                        is_exactly_a<numeric>(arg) and
                         arg.info(info_flags::positive) and
-                        ex_to<numeric>(arg) < *_num1_p;
+                        (arg-_ex1).info(info_flags::negative);
         }
         return false;
 }
@@ -85,6 +84,24 @@ static bool csc_info(const function& f, unsigned inf)
 
 static bool atan_info(const function& f, unsigned inf)
 {
+        switch (inf) {
+        case info_flags::nonnegative:
+        case info_flags::negative:
+        case info_flags::positive:
+        case info_flags::nonzero:
+                return f.op(0).info(inf);
+        }
+        return trig_info(f, inf);
+}
+
+static bool acot_info(const function& f, unsigned inf)
+{
+        switch (inf) {
+        case info_flags::nonnegative:
+        case info_flags::positive:
+        case info_flags::nonzero:
+                return true;
+        }
         return trig_info(f, inf);
 }
 
@@ -92,6 +109,7 @@ static bool sinh_info(const function& f, unsigned inf)
 {
         switch (inf) {
         case info_flags::nonnegative:
+        case info_flags::negative:
         case info_flags::positive:
         case info_flags::nonzero:
                 return f.op(0).info(inf);
@@ -102,6 +120,7 @@ static bool sinh_info(const function& f, unsigned inf)
 static bool cosh_info(const function& f, unsigned inf)
 {
         switch (inf) {
+        case info_flags::nonnegative:
         case info_flags::positive:
         case info_flags::nonzero:
                 return f.op(0).info(info_flags::real);
@@ -113,6 +132,7 @@ static bool tanh_info(const function& f, unsigned inf)
 {
         switch (inf) {
         case info_flags::nonnegative:
+        case info_flags::negative:
         case info_flags::positive:
         case info_flags::nonzero:
                 return f.op(0).info(inf);
@@ -125,6 +145,7 @@ static bool coth_info(const function& f, unsigned inf)
         switch (inf) {
         case info_flags::nonnegative:
         case info_flags::positive:
+        case info_flags::negative:
                 return f.op(0).info(inf);
         case info_flags::nonzero:
                 return true;
@@ -136,6 +157,7 @@ static bool sech_info(const function& f, unsigned inf)
 {
         switch (inf) {
         case info_flags::positive:
+        case info_flags::nonnegative:
         case info_flags::nonzero:
                 return f.op(0).info(info_flags::real);
         }
@@ -146,6 +168,7 @@ static bool csch_info(const function& f, unsigned inf)
 {
         switch (inf) {
         case info_flags::nonnegative:
+        case info_flags::negative:
         case info_flags::positive:
                 return f.op(0).info(inf);
         case info_flags::nonzero:
@@ -159,6 +182,7 @@ static bool asinh_info(const function& f, unsigned inf)
         switch (inf) {
         case info_flags::nonnegative:
         case info_flags::positive:
+        case info_flags::negative:
         case info_flags::nonzero:
                 return f.op(0).info(inf);
         }
@@ -169,6 +193,7 @@ static bool acsch_info(const function& f, unsigned inf)
 {
         switch (inf) {
         case info_flags::nonnegative:
+        case info_flags::negative:
         case info_flags::positive:
                 return f.op(0).info(inf);
         case info_flags::nonzero:
@@ -188,6 +213,9 @@ static bool gamma_info(const function& f, unsigned inf)
         case info_flags::integer:
                 return arg.info(info_flags::integer)
                    and arg.info(info_flags::positive);
+        case info_flags::even:
+                return arg.info(info_flags::integer)
+                   and (arg+_ex_2).info(info_flags::positive);
         }
         return false;
 }
@@ -208,8 +236,10 @@ static bool abs_info(const function& f, unsigned inf)
         case info_flags::real:
         case info_flags::nonnegative:
                 return true;
+        case info_flags::rational:
         case info_flags::integer:
         case info_flags::nonzero:
+        case info_flags::even:
                 return f.op(0).info(inf);
         case info_flags::positive:
                 return f.op(0).info(info_flags::nonzero);
@@ -222,8 +252,12 @@ static bool real_info(const function& f, unsigned inf)
         switch (inf) {
         case info_flags::real:
                 return true;
+        case info_flags::nonnegative:
+        case info_flags::negative:
+        case info_flags::positive:
+        case info_flags::rational:
         case info_flags::integer:
-        case info_flags::nonzero:
+        case info_flags::even:
                 return f.op(0).info(inf);
         }
         return false;
@@ -234,8 +268,6 @@ static bool imag_info(const function& f, unsigned inf)
         switch (inf) {
         case info_flags::real:
                 return true;
-        case info_flags::nonzero:
-                return f.op(0).info(info_flags::nonzero);
         }
         return false;
 }
@@ -248,6 +280,13 @@ static bool factorial_info(const function& f, unsigned inf)
         case info_flags::nonnegative:
         case info_flags::integer:
                 return arg.info(inf);
+        case info_flags::rational:
+                return arg.info(info_flags::integer);
+        case info_flags::even:
+                return (arg.info(info_flags::integer)
+                        and (arg+_ex_1).info(info_flags::positive))
+                       or (arg.info(info_flags::even)
+                        and arg.info(info_flags::positive));
         }
         return false;
 }
@@ -279,6 +318,7 @@ bool function::info(unsigned inf) const
                 {sec_SERIAL::serial, &sec_info},
                 {csc_SERIAL::serial, &csc_info},
                 {atan_SERIAL::serial, &atan_info},
+                {acot_SERIAL::serial, &acot_info},
                 {sinh_SERIAL::serial, &sinh_info},
                 {cosh_SERIAL::serial, &cosh_info},
                 {tanh_SERIAL::serial, &tanh_info},
