@@ -182,17 +182,20 @@ void infinity::do_print_python_repr(const print_python_repr & c, unsigned /*leve
 
 bool infinity::info(unsigned inf) const
 {
-	if (inf==info_flags::infinity)
-		return true;
-	if (inf == info_flags::real || 
-	    inf == info_flags::positive ||
-	    inf == info_flags::negative)
-		return direction.info(inf);
-	if (inf == info_flags::nonnegative)
-		return direction.info(info_flags::positive);
-	if (inf == info_flags::nonnegative)
-		return direction.info(info_flags::positive);
-	return inherited::info(inf);
+        switch (inf) {
+                case info_flags::infinity:
+        		return true;
+                case info_flags::real:
+                        return not direction.is_zero()
+                                and direction.info(info_flags::real);
+                case info_flags::positive:
+	        case info_flags::negative:
+        		return direction.info(inf);
+	        case info_flags::nonnegative:
+        		return direction.info(info_flags::positive);
+                default:
+                	return inherited::info(inf);
+        }
 }
 
 ex infinity::evalf(int /*level*/, PyObject* /*parent*/) const
