@@ -22,6 +22,7 @@
 
 #include "inifcns.h"
 #include "ex.h"
+#include "ex_utils.h"
 #include "constant.h"
 #include "lst.h"
 #include "fderivative.h"
@@ -31,12 +32,12 @@
 #include "relational.h"
 #include "pseries.h"
 #include "symbol.h"
-#include "symmetry.h"
 #include "utils.h"
 #include "wildcard.h"
 
 #include <vector>
 #include <stdexcept>
+#include <string>
 #include <sstream>
 
 namespace GiNaC {
@@ -291,7 +292,7 @@ static ex abs_eval(const ex & arg)
                 }
                 if (is_prod_neg)
                         prod *= _ex_1;
-                if (not prod_sy.is_integer_one())
+                if (not prod_sy.is_one())
                         prod *= abs(prod_sy).hold();
                 return prod;
 	}
@@ -317,11 +318,6 @@ static void abs_print_latex(const ex & arg, const print_context & c)
 	c.s << "{\\left| "; arg.print(c); c.s << " \\right|}";
 }
 
-static void abs_print_csrc_float(const ex & arg, const print_context & c)
-{
-	c.s << "fabs("; arg.print(c); c.s << ")";
-}
-
 static ex abs_conjugate(const ex & arg)
 {
 	return abs(arg).hold();
@@ -360,8 +356,6 @@ REGISTER_FUNCTION(abs, eval_func(abs_eval).
                        evalf_func(abs_evalf).
                        expl_derivative_func(abs_expl_derivative).
                        print_func<print_latex>(abs_print_latex).
-                       print_func<print_csrc_float>(abs_print_csrc_float).
-                       print_func<print_csrc_double>(abs_print_csrc_float).
                        derivative_func(abs_deriv).
                        conjugate_func(abs_conjugate).
                        real_part_func(abs_real_part).
@@ -723,7 +717,6 @@ REGISTER_FUNCTION(eta, eval_func(eta_eval).
                        evalf_func(eta_evalf).
                        series_func(eta_series).
                        latex_name("\\eta").
-                       set_symmetry(sy_symm(0, 1)).
                        conjugate_func(eta_conjugate).
                        real_part_func(eta_real_part).
                        imag_part_func(eta_imag_part));
@@ -755,7 +748,8 @@ static ex Order_series(const ex & x, const relational & r, int order, unsigned o
 	epvector new_seq;
 	GINAC_ASSERT(is_a<symbol>(r.lhs()));
 	const symbol &s = ex_to<symbol>(r.lhs());
-	new_seq.push_back(expair(Order(_ex1), numeric(std::min(x.ldegree(s), order))));
+        int ldeg = x.ldegree(s).to_int();
+	new_seq.push_back(expair(Order(_ex1), numeric(std::min(ldeg, order))));
 	return pseries(r, new_seq);
 }
 
@@ -863,4 +857,32 @@ REGISTER_FUNCTION(cases, eval_func(cases_eval).
                        conjugate_func(cases_conjugate).
                        real_part_func(cases_real_part).
                        imag_part_func(cases_imag_part));
+
+static std::string ex_to_str(const ex& e) {
+        std::stringstream tstream;
+        print_context c(tstream);
+        e.print(c);
+        return tstream.str();
+}
+
+static ex set_of_all_eval(const ex& arg1, const ex& arg2)
+{
+        if (is_exactly_a<function>(arg2)) {
+                std::string name = ex_to<function>(arg2).get_name();
+                if (not (name == "element_of"))
+                        throw std::runtime_error(std::string("not a set membership statement: ") + ex_to_str(arg2));
+                return set_of_all(arg1, arg2).hold();
+        }
+        if (not is_exactly_a<exprseq>(arg2))
+                throw std::runtime_error(std::string("not a set membership statement: ") + ex_to_str(arg2));
+        for (auto item : arg2) {
+                if (not is_exactly_a<function>(item)
+                    or not (ex_to<function>(item).get_name() == "element_of"))
+                        throw std::runtime_error(std::string("not a set membership statement: ") + ex_to_str(item));
+        }
+        return set_of_all(arg1, arg2).hold();
+}
+
+REGISTER_FUNCTION(set_of_all, eval_func(set_of_all_eval));
+
 } // namespace GiNaC
