@@ -25,6 +25,7 @@
 
 #include "inifcns.h"
 #include "ex.h"
+#include "ex_utils.h"
 #include "constant.h"
 #include "infinity.h"
 #include "numeric.h"
@@ -770,11 +771,11 @@ static ex acosh_eval(const ex & x)
 			return Pi*I*numeric(1,2);
 
 		// acosh(1) -> 0
-		if (x.is_equal(_ex1))
+		if (x.is_one())
 			return _ex0;
 
 		// acosh(-1) -> Pi*I
-		if (x.is_equal(_ex_1))
+		if (x.is_minus_one())
 			return Pi*I;
 
 		// acosh(float) -> float
@@ -826,10 +827,12 @@ REGISTER_FUNCTION(acosh, eval_func(acosh_eval).
 
 static ex atanh_eval(const ex & x)
 {
-	if (is_exactly_a<numeric>(x)) {
+        if (is_exactly_a<numeric>(x)) {
+
+                const numeric& num = ex_to<numeric>(x);
 
 		// atanh(0) -> 0
-		if (x.is_zero())
+		if (num.is_zero())
 			return _ex0;
 
 		/*
@@ -839,19 +842,28 @@ static ex atanh_eval(const ex & x)
 		*/
 
 		// atanh(1) -> oo
-		if (x.is_equal(_ex1))
+		if (num.is_one())
 			return Infinity;
 		// atahn(-1) -> -oo
-		if (x.is_equal(_ex_1))
+		if (num.is_minus_one())
 			return NegInfinity;
 
-		// atanh(float) -> float
-		if (x.info(info_flags::inexact))
-			return atanh(ex_to<numeric>(x));
+                // atanh(real) outside (-1,1)
+                if (num.is_real() and abs(num) > *_num1_p)
+                        return NaN;
 
 		// atanh() is odd
-		if (x.info(info_flags::negative))
+		if (num.is_negative())
 			return -atanh(-x);
+
+		// atanh(float) -> float
+		if (not num.is_exact())
+			return atanh(num);
+                
+                if (num.is_integer() or num.is_rational())
+                        return _ex1_2 * log(ex((*_num1_p + num)/(*_num1_p - num)));
+
+                return atanh(x).hold();
 	}
 	
 	// atanh(oo) -> -i*pi/2
@@ -944,18 +956,24 @@ REGISTER_FUNCTION(atanh, eval_func(atanh_eval).
 static ex acoth_eval(const ex & x)
 {
         if (is_exactly_a<numeric>(x)) {
+                const numeric& num = ex_to<numeric>(x);
                 // acoth(1) -> oo
-                if (x.is_equal(_ex1))
+                if (num.is_one())
                         return Infinity;
                 // acoth(-1) -> -oo
-                if (x.is_equal(_ex_1))
+                if (num.is_minus_one())
                         return NegInfinity;
                 //acoth(float) -> float 
-                if (x.info(info_flags::inexact))
-                        return atanh(ex_to<numeric>(x).inverse());
+                if (not num.is_exact())
+                        return atanh(num.inverse());
+                // acoth(real) inside (-1,1)
+                if (num.is_real() and abs(num) < *_num1_p)
+                        return NaN;
                 // acoth() is odd
-                if (x.info(info_flags::negative))
-                        return -acoth(-x);
+                if (num.is_negative())
+                        return -acoth(num.negative());
+                if (num.is_integer() or num.is_rational())
+                        return _ex1_2 * log(ex((num + *_num1_p)/(num - *_num1_p)));
         }
        
 	if (is_exactly_a<function>(x)) {
@@ -1071,10 +1089,10 @@ static ex asech_eval(const ex & x)
                 if (x.is_zero())
                         return Infinity;
                 // asech(1) -> 0
-                if (x.is_equal(_ex1))
+                if (x.is_one())
                         return _ex0;
                 //asech(-1) -> I*Pi
-                if (x.is_equal(_ex_1))
+                if (x.is_minus_one())
                         return Pi*I;
                 //asech(float) -> float 
                 if (x.info(info_flags::inexact))
