@@ -23,8 +23,6 @@
 #include "matrix.h"
 #include "numeric.h"
 #include "lst.h"
-#include "idx.h"
-#include "indexed.h"
 #include "add.h"
 #include "power.h"
 #include "symbol.h"
@@ -208,7 +206,7 @@ ex & matrix::let_op(size_t i)
 ex matrix::eval(int level) const
 {
 	// check if we have to do anything at all
-	if ((level==1)&&((flags & status_flags::evaluated) != 0u))
+	if (level == 1 and is_evaluated())
 		return *this;
 	
 	// emergency break
@@ -318,6 +316,7 @@ bool matrix::match_same_type(const basic & other) const
 	return row == o.rows() && col == o.cols();
 }
 
+#if 0
 /** Automatic symbolic evaluation of an indexed matrix. */
 ex matrix::eval_indexed(const basic & i) const
 {
@@ -543,7 +542,7 @@ bool matrix::contract_with(exvector::iterator self, exvector::iterator other, ex
 
 	return false;
 }
-
+#endif
 
 //////////
 // non-virtual functions in this class
@@ -1053,11 +1052,11 @@ matrix matrix::solve(const matrix & vars,
 		unsigned last_assigned_sol = n+1;
 		for (int r=mm-1; r>=0; --r) {
 			unsigned fnz = 1;    // first non-zero in row
-			while ((fnz<=n) && (aug.m[r*(n+p)+(fnz-1)].is_zero()))
+			while ((fnz<=n) && (aug.m[r*(n+p)+(fnz-1)].normal().is_zero()))
 				++fnz;
 			if (fnz>n) {
 				// row consists only of zeros, corresponding rhs must be 0, too
-				if (!aug.m[r*(n+p)+n+co].is_zero()) {
+				if (!aug.m[r*(n+p)+n+co].normal().is_zero()) {
 					throw (std::runtime_error("matrix::solve(): inconsistent linear system"));
 				}
 			} else {
@@ -1443,7 +1442,9 @@ int matrix::fraction_free_elimination(const bool det)
 					                    tmp_n.m[r2*n+c], true);
 					check &= static_cast<int>(divide(dividend_d, divisor_d,
 					                tmp_d.m[r2*n+c], true));
+#ifdef DO_GINAC_ASSERT
 					GINAC_ASSERT(check);
+#endif
 				}
 				// fill up left hand side with zeros
 				for (unsigned c=r0; c<=c0; ++c)
