@@ -50,7 +50,6 @@ namespace GiNaC {
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(mul, expairseq,
   print_func<print_context>(&mul::do_print).
   print_func<print_latex>(&mul::do_print_latex).
-  print_func<print_csrc>(&mul::do_print_csrc).
   print_func<print_tree>(&mul::do_print_tree).
   print_func<print_python_repr>(&mul::do_print_python_repr))
 
@@ -378,61 +377,6 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 
 }
 
-void mul::do_print_csrc(const print_csrc & c, unsigned level) const
-{
-	if (precedence() <= level)
-		c.s << "(";
-
-	if (not overall_coeff.is_one()) {
-		if (overall_coeff.is_minus_one()) {
-			c.s << "-";
-		}
-		else {
-			overall_coeff.print(c, precedence());
-			c.s << "*";
-		}
-	}
-
-	// Print arguments, separated by "*" or "/"
-        bool seqstart = true;
-	for (const auto & item : seq) {
-                const numeric co = ex_to<numeric>(item.coeff);
-		// If the first argument is a negative integer power, it gets printed as "1.0/<expr>"
-		bool needclosingparenthesis = false;
-		if (seqstart and co.info(info_flags::negint)) {
-			if (is_a<print_csrc_cl_N>(c)) {
-				c.s << "recip(";
-				needclosingparenthesis = true;
-			} else
-				c.s << "1.0/";
-                        seqstart = false;
-		}
-
-		// If the exponent is 1 or -1, it is left out
-		if (co.is_one() or co.is_minus_one()) {
-			item.rest.print(c, precedence());
-		}
-		else if (co.info(info_flags::negint))
-			// Outer parens around ex needed for broken GCC parser:
-			(ex(power(item.rest, -co))).print(c, level);
-		else
-			// Outer parens around ex needed for broken GCC parser:
-			(ex(power(item.rest, co))).print(c, level);
-
-		if (needclosingparenthesis)
-			c.s << ")";
-
-		// Separator is "/" for negative integer powers, "*" otherwise
-                if (co.info(info_flags::negint))
-                        c.s << "/";
-                else
-                        c.s << "*";
-	}
-
-	if (precedence() <= level)
-		c.s << ")";
-}
-
 void mul::do_print_python_repr(const print_python_repr & c, unsigned level) const
 {
 	c.s << class_name() << '(';
@@ -447,6 +391,9 @@ void mul::do_print_python_repr(const print_python_repr & c, unsigned level) cons
 bool mul::info(unsigned inf) const
 {
 	switch (inf) {
+		case info_flags::nonzero:
+                        return info(info_flags::positive)
+                                or info(info_flags::negative);
 		case info_flags::polynomial:
 		case info_flags::integer_polynomial:
 		case info_flags::cinteger_polynomial:
@@ -456,8 +403,6 @@ bool mul::info(unsigned inf) const
 		case info_flags::integer:
 		case info_flags::crational:
 		case info_flags::cinteger:
-                case info_flags::nonzero:
-		case info_flags::even:
 		case info_flags::crational_polynomial:
 		case info_flags::rational_function: {
                         if (not overall_coeff.info(info_flags::real))
@@ -469,6 +414,18 @@ bool mul::info(unsigned inf) const
 				return true;
 			return overall_coeff.info(inf);
 		}
+		case info_flags::even:
+                {
+                        bool even_seen = false;
+                        for (const auto & elem : seq) {
+				const ex& e = recombine_pair_to_ex(elem);
+                                if (not e.info(info_flags::integer))
+					return false;
+                                if (e.info(info_flags::even))
+                                        even_seen = true;
+                        }
+                        return even_seen or overall_coeff.is_even();
+                }
 		case info_flags::inexact:
 		case info_flags::algebraic: {
                         if (overall_coeff.info(inf))
