@@ -53,7 +53,6 @@ namespace GiNaC {
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(power, basic,
   print_func<print_dflt>(&power::do_print_dflt).
   print_func<print_latex>(&power::do_print_latex).
-  print_func<print_csrc>(&power::do_print_csrc).
   print_func<print_python>(&power::do_print_python).
   print_func<print_python_repr>(&power::do_print_python_repr))
 
@@ -237,53 +236,6 @@ static void print_sym_pow(const print_context & c, const symbol &x, int exp)
 	}
 }
 
-void power::do_print_csrc(const print_csrc & c, unsigned level) const
-{
-	if (is_a<print_csrc_cl_N>(c)) {
-		if (exponent.is_one()) {
-			c.s << "recip(";
-			basis.print(c);
-			c.s << ')';
-			return;
-		}
-		c.s << "expt(";
-		basis.print(c);
-		c.s << ", ";
-		exponent.print(c);
-		c.s << ')';
-		return;
-	}
-
-	// Integer powers of symbols are printed in a special, optimized way
-	if (is_exactly_a<numeric>(exponent)
-            and exponent.info(info_flags::integer)
-	    and (is_exactly_a<symbol>(basis) or is_exactly_a<constant>(basis))) {
-		int exp = ex_to<numeric>(exponent).to_int();
-		if (exp > 0)
-			c.s << '(';
-		else {
-			exp = -exp;
-			c.s << "1.0/(";
-		}
-		print_sym_pow(c, ex_to<symbol>(basis), exp);
-		c.s << ')';
-
-	// <expr>^-1 is printed as "1.0/<expr>" or with the recip() function of CLN
-	} else if (exponent.is_one()) {
-		c.s << "1.0/(";
-		basis.print(c);
-		c.s << ')';
-
-	// Otherwise, use the pow() function
-	} else {
-		c.s << "pow(";
-		basis.print(c);
-		c.s << ',';
-		exponent.print(c);
-		c.s << ')';
-	}
-}
-
 void power::do_print_python(const print_python & c, unsigned level) const
 {
 	print_power(c, "**", "", "", level);
@@ -307,9 +259,13 @@ bool power::info(unsigned inf) const
 		case info_flags::rational_polynomial:
 		case info_flags::crational_polynomial:
                 case info_flags::integer:
-			return exponent.info(info_flags::nonnegint) &&
-			       basis.info(inf);
+			return exponent.info(info_flags::nonnegint)
+			        and basis.info(inf);
+                case info_flags::even:
+			return exponent.info(info_flags::posint)
+                                and basis.info(info_flags::integer);
 		case info_flags::rational_function:
+                case info_flags::rational:
 			return exponent.info(info_flags::integer) &&
 			       basis.info(inf);
                 case info_flags::inexact:
@@ -330,11 +286,19 @@ bool power::info(unsigned inf) const
 		case info_flags::nonnegative:
 			return (basis.info(info_flags::positive) && exponent.info(info_flags::real)) ||
 			       (basis.info(info_flags::real) && exponent.info(info_flags::integer) && exponent.info(info_flags::even));
+		case info_flags::negative:
+                        if (exponent.info(info_flags::odd))
+                                return basis.info(inf);
+                        return false;
 		case info_flags::real:
 			return ((basis.info(inf) and
                                  (exponent.info(info_flags::integer))) or
                                 (basis.info(info_flags::positive) and
                                  exponent.info(inf)));
+                case info_flags::nonzero:
+                        return (basis.info(inf)
+                                or exponent.is_zero()
+                                or exponent.info(info_flags::negative));
 	}
 	return inherited::info(inf);
 }
